c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine dump_stresses
#include "paradyn.h"
#include "mpif.h"
      integer istatus(mpi_status_size)
 900  format (i7,6f10.4)
 901  format (6f12.4)

#ifdef STRESS

      if (node.eq.0) write (6,*) 'Dumping Stresses ...'

      if (natoms.eq.0) call error('No atoms defined')

      call mpi_barrier(mpi_comm_world,ierror)
      time1 = mpi_wtime()
      
      if (idumpstrsflag.eq.0.and.node.eq.0) then
        open (unit=4,file=dumpstrsfile)
        close (4,status='delete')
        open (unit=4,file=dumpstrsfile)
      endif
      idumpstrsflag = 1
      
c accumulate global stress

      if (ibdtyp.eq.1) call global_stress

c accumulate total stress on each atom

      call scatter(strsrow,strs,plan_row6)
      if (newton.eq.2) then
        call scatter(strscol,strstmp,plan_col6)
        do i = 1,nlocal
          strs(1,i) = strs(1,i) + strstmp(1,i)
          strs(2,i) = strs(2,i) + strstmp(2,i)
          strs(3,i) = strs(3,i) + strstmp(3,i)
          strs(4,i) = strs(4,i) + strstmp(4,i)
          strs(5,i) = strs(5,i) + strstmp(5,i)
          strs(6,i) = strs(6,i) + strstmp(6,i)
        enddo
      endif

c add in kinetic energy term for each atom

      do i = 1,nlocal
        ity = itype(i)
        strs(1,i) = strs(1,i) + amass(ity)*v(1,i)*v(1,i)
        strs(2,i) = strs(2,i) + amass(ity)*v(2,i)*v(2,i)
        strs(3,i) = strs(3,i) + amass(ity)*v(3,i)*v(3,i)
        strs(4,i) = strs(4,i) + amass(ity)*v(1,i)*v(2,i)
        strs(5,i) = strs(5,i) + amass(ity)*v(1,i)*v(3,i)
        strs(6,i) = strs(6,i) + amass(ity)*v(2,i)*v(3,i)
      enddo

c write to file

      if (node.ne.0) then

        call mpi_recv(ntmp,0,mpi_integer,0,0,
     $       mpi_comm_world,istatus,ierror)
        call mpi_send(tag,nlocal,mpi_integer,0,0,
     $       mpi_comm_world,ierror)
        call mpi_send(strs,6*nlocal,mpi_double_precision,
     $       0,0,mpi_comm_world,ierror)

      else
        
        write (4,*) 'ITEM: NUMBER OF ATOMS'
        write (4,*) natoms
        write (4,*) 'ITEM: GLOBAL STRESS'
        write (4,901) stress(1),stress(2),stress(3),
     $       stress(4),stress(5),stress(6)
        write (4,*) 'ITEM: TIME'
        write (4,*) time_stamp
        write (4,*) 'ITEM: STRESSES'

        do i = 1,nlocal
          write (4,900) tag(i),strs(1,i),strs(2,i),strs(3,i),
     $         strs(4,i),strs(5,i),strs(6,i)
        enddo
        
        do iproc = 1,nprocs-1
          call mpi_send(ntmp,0,mpi_integer,iproc,0,
     $         mpi_comm_world,ierror)
          call mpi_recv(tagtmp,maxlocal,mpi_integer,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          call mpi_get_count(istatus,mpi_integer,icnt,ierror)
          call mpi_recv(strstmp,6*icnt,mpi_double_precision,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          do i = 1,icnt
            write (4,900) tagtmp(i),
     $           strstmp(1,i),strstmp(2,i),strstmp(3,i),
     $           strstmp(4,i),strstmp(5,i),strstmp(6,i)
          enddo
        enddo
        
      endif

      call mpi_barrier(mpi_comm_world,ierror)
      time2 = mpi_wtime()
      time_io = time_io + time2-time1
      
#endif

      return
      end
