c Parallel Molecular Dynamics Algorithm & Benchmark
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c See the README file for more information
c

c robust wrapper on malloc
c itype = 0 -> integer memory
c itype = 1 -> double precision memory

      subroutine memalloc(ptr,n,itype)
      pointer (ptr,x(1))
      integer n,itype

      parameter (ibyte=4,nbyte=8)
      integer memtotal
      common /memory/ memtotal
      data memtotal /0/

      if (n.eq.0) then
        ptr = 0
        return
      endif

#ifdef CRAY
      call hpalloc(ptr,n,ierr,0)
      if (ierr.gt.0) ptr = 0
      memtotal = memtotal + nbyte*n
#else
      if (itype.eq.0) then
        ptr = malloc(ibyte*n)
        memtotal = memtotal + ibyte*n
      else
        ptr = malloc(nbyte*n)
        memtotal = memtotal + nbyte*n
      endif
#endif

      if (ptr.eq.0) then
        write (6,*) 'ERROR: Could not malloc words =',n
        call exit(0)
      endif

      return
      end


c robust wrapper on free

      subroutine memfree(ptr)
      pointer (ptr,x(1))

#ifdef CRAY
      if (ptr.ne.0) call hpdeallc(ptr,ierr,0)
#else
      if (ptr.ne.0) call free(ptr)
#endif

      return
      end


c reset memory counter

      subroutine memset(n)
      integer n

      integer memtotal
      common /memory/ memtotal

      memtotal = n

      return
      end


c query memory counter

      subroutine memquery(n)
      integer n

      integer memtotal
      common /memory/ memtotal

      n = memtotal

      return
      end
