c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine gneigh(i)
#include "paradyn.h"
      
      xi = xrow(1,i)
      yi = xrow(2,i)
      zi = xrow(3,i)
      m = nlarge_pt(i-1)
      n = nsmall_pt(i-1)
      
      do k = 1,nlarge_pt(i)-nlarge_pt(i-1)
        m = m + 1
        j = nlarge_lst(m)
        dx = xi - xcol(1,j)
        dx = dx - perlen(1)*nint(dx/perlen(1))
        dy = yi - xcol(2,j)
        dy = dy - perlen(2)*nint(dy/perlen(2))
        dz = zi - xcol(3,j)
        dz = dz - perlen(3)*nint(dz/perlen(3))
        rsq = dx*dx + dy*dy + dz*dz
        if (rsq.le.cutsq1) then
          n = n + 1
          rneigh(n) = sqrt(rsq)
          jneigh(n) = j
          dneigh(1,n) = dx
          dneigh(2,n) = dy
          dneigh(3,n) = dz
        endif
      enddo
      
      if (n.gt.maxnsmall*maxlocal) then
        write (6,*) 'Actual neighbors too big - boost maxnsmall:',
     $       node,i,n
        call exit(0)
      endif
      
      nsmall_pt(i) = n
      
      return
      end
      
      
c neighbor driver

      subroutine neighbor
#include "paradyn.h"
#include "mpif.h"
      
      if (newlst.eq.0) then
        
        drsq = 0.0D0
        do i = 1,nlocal
          dx = xold(1,i) - x(1,i)
          dx = dx - perlen(1)*nint(dx/perlen(1))
          dy = xold(2,i) - x(2,i)
          dy = dy - perlen(2)*nint(dy/perlen(2))
          dz = xold(3,i) - x(3,i)
          dz = dz - perlen(3)*nint(dz/perlen(3))
          rsq = dx*dx + dy*dy + dz*dz
          drsq = max(drsq,rsq)
        enddo
        
        call mpi_allreduce(drsq,tmp,1,mpi_double_precision,mpi_max,
     $       mpi_comm_world,ierror)
        if (tmp.le.trigger) return
        
      endif
      
      do i = 1,nlocal
        xold(1,i) = x(1,i)
        xold(2,i) = x(2,i)
        xold(3,i) = x(3,i)
      enddo
      
      newlst = 0
      nneigh = nneigh + 1
      
      if (newton.eq.1) then
        if (nmeth.eq.1) then
          call neighbor0
        else
          call neighbor1
        endif
      else
        if (nmeth.eq.1) then
          call neighbor2
        else
          call neighbor3
        endif
      endif
      
      mxnlarge = max(mxnlarge,nlarge_pt(nrow))
      avenlarge = avenlarge + nlarge_pt(nrow)

      return
      end
      

c no Newton's law, N^2 search
      
      subroutine neighbor0
#include "paradyn.h"
      
      npnt = 0
      do i = 1,nrow
        itag = tagrow(i)
        xi = xrow(1,i)
        yi = xrow(2,i)
        zi = xrow(3,i)
        do j = 1,ncol
          jtag = tagcol(j)
          dx = xi - xcol(1,j)
          dx = dx - idynper(1)*perlen(1)*nint(dx/perlen(1))
          dy = yi - xcol(2,j)
          dy = dy - idynper(2)*perlen(2)*nint(dy/perlen(2))
          dz = zi - xcol(3,j)
          dz = dz - idynper(3)*perlen(3)*nint(dz/perlen(3))
          rsq = dx*dx + dy*dy + dz*dz
          if (rsq.le.cutsq2.and.itag.ne.jtag) then
            npnt = npnt + 1
            nlarge_lst(npnt) = j
          endif
        enddo
        nlarge_pt(i) = npnt
        if (npnt.gt.maxnlarge*maxlocal) then
          write (6,*) 'Neighbor list too big - boost maxnlarge:',
     $         node,i,npnt
          call exit(0)
        endif
      enddo
      
      return
      end
      
      
c no Newton's law, binned search

      subroutine neighbor1
#include "paradyn.h"
      
c bin column atoms

      call setbins
      
      do i = 1,nbinx*nbiny*nbinz
        binpnt(i) = 0
      enddo
      
      do i = 1,ncol
        ix = (xcol(1,i) - actlb(1)) / binsizex
        iy = (xcol(2,i) - actlb(2)) / binsizey
        iz = (xcol(3,i) - actlb(3)) / binsizez
        ib = iz*nbiny*nbinx + iy*nbinx + ix + 1
        bin(i) = binpnt(ib)
        binpnt(ib) = i
      enddo
      
      npnt = 0
      do i = 1,nrow
        itag = tagrow(i)
        xi = xrow(1,i)
        yi = xrow(2,i)
        zi = xrow(3,i)
        ixx = (xi - actlb(1)) / binsizex
        iyy = (yi - actlb(2)) / binsizey
        izz = (zi - actlb(3)) / binsizez
        do k = 0,26
          ix = ixx + mod(k,3) - 1
          if (ix.lt.0) ix = nbinx - 1
          if (ix.eq.nbinx) ix = 0
          iy = iyy + mod(k/3,3) - 1
          if (iy.lt.0) iy = nbiny - 1
          if (iy.eq.nbiny) iy = 0
          iz = izz + k/9 - 1
          if (iz.lt.0) iz = nbinz - 1
          if (iz.eq.nbinz) iz = 0
          ib = iz*nbiny*nbinx + iy*nbinx + ix + 1
          j = binpnt(ib)
 10       if (j.ne.0) then
            jtag = tagcol(j)
            dx = xi - xcol(1,j)
            dx = dx - idynper(1)*perlen(1)*nint(dx/perlen(1))
            dy = yi - xcol(2,j)
            dy = dy - idynper(2)*perlen(2)*nint(dy/perlen(2))
            dz = zi - xcol(3,j)
            dz = dz - idynper(3)*perlen(3)*nint(dz/perlen(3))
            rsq = dx*dx + dy*dy + dz*dz
            if (rsq.le.cutsq2.and.itag.ne.jtag) then
              npnt = npnt + 1
              nlarge_lst(npnt) = j
            endif
            j = bin(j)
            goto 10
          endif
        enddo
        nlarge_pt(i) = npnt
        if (npnt.gt.maxnlarge*maxlocal) then
          write (6,*) 'Neighbor list too big - boost maxnlarge:',
     $         node,i,npnt
          call exit(0)
        endif
      enddo
      
      return
      end
      
      
c Newton's law, N^2 search

      subroutine neighbor2
#include "paradyn.h"
      
      npnt = 0
      do i = 1,nrow
        itag = tagrow(i)
        xi = xrow(1,i)
        yi = xrow(2,i)
        zi = xrow(3,i)
        do j = 1,ncol
          jtag = tagcol(j)
          if (itag.le.jtag) then
            if (mod(itag+jtag,2).eq.0) goto 10
          else
            if (mod(itag+jtag,2).eq.1) goto 10
          endif
          dx = xi - xcol(1,j)
          dx = dx - idynper(1)*perlen(1)*nint(dx/perlen(1))
          dy = yi - xcol(2,j)
          dy = dy - idynper(2)*perlen(2)*nint(dy/perlen(2))
          dz = zi - xcol(3,j)
          dz = dz - idynper(3)*perlen(3)*nint(dz/perlen(3))
          rsq = dx*dx + dy*dy + dz*dz
          if (rsq.le.cutsq2) then
            npnt = npnt + 1
            nlarge_lst(npnt) = j
          endif
 10     enddo
        nlarge_pt(i) = npnt
        if (npnt.gt.maxnlarge*maxlocal) then
          write (6,*) 'Neighbor list too big - boost maxnlarge:',
     $         node,i,npnt
          call exit(0)
        endif
      enddo
      
      return
      end
      
      
c Newton's law, binned search

      subroutine neighbor3
#include "paradyn.h"

c bin column atoms

      call setbins
      
      do i = 1,nbinx*nbiny*nbinz
        binpnt(i) = 0
      enddo

      do i = 1,ncol
        ix = (xcol(1,i) - actlb(1)) / binsizex
        iy = (xcol(2,i) - actlb(2)) / binsizey
        iz = (xcol(3,i) - actlb(3)) / binsizez
        ib = iz*nbiny*nbinx + iy*nbinx + ix + 1
        bin(i) = binpnt(ib)
        binpnt(ib) = i
      enddo

      npnt = 0
      do i = 1,nrow
        itag = tagrow(i)
        xi = xrow(1,i)
        yi = xrow(2,i)
        zi = xrow(3,i)
        ixx = (xi - actlb(1)) / binsizex
        iyy = (yi - actlb(2)) / binsizey
        izz = (zi - actlb(3)) / binsizez
        do k = 0,26
          ix = ixx + mod(k,3) - 1
          if (ix.lt.0) ix = nbinx - 1
          if (ix.eq.nbinx) ix = 0
          iy = iyy + mod(k/3,3) - 1
          if (iy.lt.0) iy = nbiny - 1
          if (iy.eq.nbiny) iy = 0
          iz = izz + k/9 - 1
          if (iz.lt.0) iz = nbinz - 1
          if (iz.eq.nbinz) iz = 0
          ib = iz*nbiny*nbinx + iy*nbinx + ix + 1
          j = binpnt(ib)
 10       if (j.ne.0) then
            jtag = tagcol(j)
            if (itag.le.jtag) then
              if (mod(itag+jtag,2).eq.0) goto 20
            else
              if (mod(itag+jtag,2).eq.1) goto 20
            endif
            dx = xi - xcol(1,j)
            dx = dx - idynper(1)*perlen(1)*nint(dx/perlen(1))
            dy = yi - xcol(2,j)
            dy = dy - idynper(2)*perlen(2)*nint(dy/perlen(2))
            dz = zi - xcol(3,j)
            dz = dz - idynper(3)*perlen(3)*nint(dz/perlen(3))
            rsq = dx*dx + dy*dy + dz*dz
            if (rsq.le.cutsq2) then
              npnt = npnt + 1
              nlarge_lst(npnt) = j
            endif
 20         j = bin(j)
            goto 10
          endif
        enddo
        nlarge_pt(i) = npnt
        if (npnt.gt.maxnlarge*maxlocal) then
          write (6,*) 'Neighbor list too big - boost maxnlarge:',
     $         node,i,npnt
          call exit(0)
        endif
      enddo
      
      return
      end


c setup neighbor bins
c  if idynper = 0 in any dimension, scan all atoms for actual boundary
c  addding small -> bins slightly larger
c  prevents round-off error when atoms are binned

      subroutine setbins
#include "paradyn.h"
#include "mpif.h"
      parameter (small=1.0D-4)

      if (idynper(1).eq.0.or.idynper(2).eq.0.or.idynper(3).eq.0) then

        actbuf(1) = 1.0D20
        actbuf(2) = 1.0D20
        actbuf(3) = 1.0D20
        actbuf(4) = -1.0D20
        actbuf(5) = -1.0D20
        actbuf(6) = -1.0D20
        do i = 1,nlocal
          if (x(1,i).lt.actbuf(1)) actbuf(1) = x(1,i)
          if (x(2,i).lt.actbuf(2)) actbuf(2) = x(2,i)
          if (x(3,i).lt.actbuf(3)) actbuf(3) = x(3,i)
          if (x(1,i).gt.actbuf(4)) actbuf(4) = x(1,i)
          if (x(2,i).gt.actbuf(5)) actbuf(5) = x(2,i)
          if (x(3,i).gt.actbuf(6)) actbuf(6) = x(3,i)
        enddo

        call mpi_allreduce(actbuf,actlb,3,mpi_double_precision,
     $       mpi_min,mpi_comm_world,ierror)
        call mpi_allreduce(actbuf(4),actlen,3,mpi_double_precision,
     $       mpi_max,mpi_comm_world,ierror)

        actlen(1) = actlen(1) - actlb(1)
        actlen(2) = actlen(2) - actlb(2)
        actlen(3) = actlen(3) - actlb(3)

      endif

      if (idynper(1).eq.1) actlen(1) = perlen(1)
      if (idynper(2).eq.1) actlen(2) = perlen(2)
      if (idynper(3).eq.1) actlen(3) = perlen(3)
      nbinx = actlen(1) / sqrt(cutsq2)
      nbiny = actlen(2) / sqrt(cutsq2)
      nbinz = actlen(3) / sqrt(cutsq2)
      binsizex = (actlen(1)+small*actlen(1)) / nbinx
      binsizey = (actlen(2)+small*actlen(2)) / nbiny
      binsizez = (actlen(3)+small*actlen(3)) / nbinz
      if (idynper(1).eq.1) actlb(1) = perlb(1)
      if (idynper(2).eq.1) actlb(2) = perlb(2)
      if (idynper(3).eq.1) actlb(3) = perlb(3)

      if (nbinx.lt.3.or.nbiny.lt.3.or.nbinz.lt.3) then
        if (node.eq.0) write (6,*) 'Nbin xyz =',nbinx,nbiny,nbinz
        call error('Too few bins')
      endif
      if (nbinx*nbiny*nbinz.gt.maxbin) then
        if (node.eq.0) write (6,*) 'Nbin xyz =',nbinx,nbiny,nbinz,
     $       maxbin
        call error('Too many bins - boost maxbin')
      endif

      return
      end
